﻿using System.Linq;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Managers;
using ff14bot.Objects;
using TreeSharp;

namespace Heathcliff
{
    internal class DarkKnightRoutine : CombatRoutine
    {
        private const uint HardSlash = 3617;
        private const uint SpinningSlash = 3619;
        private const uint Scourge = 3620;
        private const uint SyphonStrike = 3623;
        private const int DarksideAura = 751;
        private const int GritAura = 743;
        private const int ScourgeAura = 741;
        private const uint BloodWeapon = 3625;
        private const uint Reprisal = 3626;
        private const uint PowerSlash = 3627;
        private const uint Convalescense = 12;
        private const uint BloodPrice = 3631;
        private const uint Souleater = 3632;
        private const uint Bloodbath = 34;
        private const uint DarkPassenger = 3633;
        private const uint Foresight = 32;

        public override ClassJobType Job
        {
            get { return ClassJobType.Paladin; }
        }

        protected override Composite CreateBehavior()
        {
            var hardSlash = Spells.CastOnTarget(HardSlash);
            var souleater = new Decorator(c => UseSouleater, Spells.CastOnTarget(Souleater));
            var syphonStrike = new Decorator(c => UseSyphonStrike, Spells.CastOnTarget(SyphonStrike));
            var spinningSlash = new Decorator(c => UseSpinningSlash, Spells.CastOnTarget(SpinningSlash));
            var powerSlash = new Decorator(c => UsePowerSlash, Spells.CastOnTarget(PowerSlash));
            var convalescense = new Decorator(c => UseCovalescense, Spells.CastOnSelf(Convalescense));
            var scourge = new Decorator(c => UseScourge, Spells.CastOnTarget(Scourge));
            var bloodweapon = Spells.CastOnSelf(BloodWeapon);
            var reprisal = Spells.CastOnTarget(Reprisal);
            var bloodPrice = new Decorator(c => UseBloodPrice, Spells.CastOnSelf(BloodPrice));
            var darkPassenger = new Decorator(c => UseDarkPassenger, Spells.CastOnTarget(DarkPassenger));
            var bloodbath = new Decorator(c => UseBloodbath, Spells.CastOnSelf(Bloodbath));
            var foresight = new Decorator(c => UseForesight, Spells.CastOnSelf(Foresight));

            var rotation = new PrioritySelector(bloodweapon, foresight, convalescense, bloodbath, reprisal, bloodPrice, powerSlash,
                souleater, darkPassenger, syphonStrike, spinningSlash, scourge, hardSlash);

            return new Decorator(c => InCombat, rotation);
        }

        private static bool InCombat
        {
            get { return Core.Me.InCombat && Core.Me.CurrentTarget is BattleCharacter; }
        }

        private static bool UseForesight
        {
            get { return Core.Me.CurrentHealthPercent < 40; }
        }

        private static bool UseDarkPassenger
        {
            get
            {
                if (Core.Me.CurrentTarget == null || !Actionmanager.CanCast(DarkPassenger, Core.Me.CurrentTarget)) { return false; }

                float heading = Core.Me.Heading;
                var location = Core.Me.Location;

                int hits = GameObjectManager.Attackers
                    .Where(a => a.InCombat && Utilities.IsFacing(location, heading, a.Location))
                    .Count(a => a.Distance(Core.Me.Location) < 15f);

                return hits > 2;
            }
        }

        private static bool UseBloodbath
        {
            get { return Core.Me.CurrentHealthPercent < 50; }
        }

        private static bool UseSouleater
        {
            get { return Actionmanager.LastSpellId == SyphonStrike; }
        }

        private static bool UseBloodPrice
        {
            get { return Core.Me.CurrentManaPercent < 100; }
        }

        private static bool UseScourge
        {
            get
            {
                var target = Core.Me.CurrentTarget as BattleCharacter;
                return target != null && !target.HasMyAura(ScourgeAura);
            }
        }

        private static bool UseCovalescense
        {
            get { return Core.Me.CurrentHealthPercent < 50; }
        }

        private static bool UseSyphonStrike
        {
            get 
            { 
                return (Core.Me.CurrentManaPercent < 60 || UseSouleaterCombo) 
                    && Actionmanager.LastSpellId == HardSlash; 
            }
        }

        private static bool UseSouleaterCombo
        {
            get
            {
                return Actionmanager.HasSpell(Souleater) && Core.Me.CurrentHealthPercent < 60
                    && Core.Me.HasAura(GritAura);
            }
        }

        private static bool UseSpinningSlash
        {
            get { return Actionmanager.LastSpellId == HardSlash; }
        }

        private static bool UsePowerSlash
        {
            get { return Actionmanager.LastSpellId == SpinningSlash; }
        }
    }
}
